"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Form = void 0;
const formControl_1 = require("./formControl");
class Form {
    constructor(_controls = {}) {
        this._formatControls = (formControls = {}) => {
            const controls = Object.create({});
            try {
                Object.keys(formControls).forEach((key) => {
                    const { value, validators = [], touched } = formControls[key];
                    const control = new formControl_1.FormControl(formControls, value, validators, touched);
                    controls[key] = control;
                });
                return controls;
            }
            catch (error) {
                throw new Error(`Invalid form format: ${error.message}`);
            }
        };
        this._form = {
            controls: this._formatControls(_controls),
            valid: this._isValidForm(_controls),
            value: this._getValues(_controls),
        };
    }
    get controls() {
        return this._form.controls;
    }
    get valid() {
        return this._form.valid;
    }
    get value() {
        return this._form.value;
    }
    _isValidForm(_controls) {
        const isValid = !Object.keys(_controls).find((prop) => _controls[prop].validators.some((validator) => validator.validatorfunction.call(_controls, _controls[prop].value) === false));
        return isValid;
    }
    _getValues(_controls) {
        return Object.keys(_controls).reduce((obj, prop) => {
            obj[prop] = _controls[prop].value;
            return obj;
        }, {});
    }
}
exports.Form = Form;
