"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useForm = void 0;
const formControl_1 = require("./formControl");
const react_1 = require("react");
const form_1 = require("./form");
const proxykits_1 = require("./formkits.js/proxykits");
exports.useForm = (controls) => {
    const [form, setForm] = react_1.useState(new form_1.Form(controls));
    const updateForm = (formControls) => {
        setForm((prevState) => {
            const updatedControls = Object.assign(Object.assign({}, prevState.controls), formControls);
            return new form_1.Form(updatedControls);
        });
    };
    const handleControlEvent = (event) => {
        const { value, name, type, checked } = event.target;
        if (name) {
            const validators = form.controls[name].validators;
            const _value = type === 'checkbox' ? checked : value;
            const control = new formControl_1.FormControl(form.controls, _value, validators, true);
            updateForm({ [name]: control });
        }
        else {
            throw new Error('Missing property name. <input *name="prop_name"/>');
        }
    };
    const setFormControlValue = (key, value) => {
        const validators = form.controls[key].validators;
        updateForm({ [key]: new formControl_1.FormControl(form.controls, value, validators, true) });
    };
    const resetForm = () => {
        updateForm(Object.assign({}, controls));
    };
    const addFormControl = (name, control) => {
        updateForm({ [name]: control });
    };
    const removeFormControl = (name) => {
        const currentControls = Object.assign({}, form.controls);
        delete currentControls[name];
        try {
            setForm(new form_1.Form(currentControls));
        }
        catch (error) {
            throw new Error(`Unable to remove control. Maybe has dependency relation with other controls: ${error}`);
        }
    };
    return {
        form: new Proxy(form, formHandler),
        handleControlEvent,
        setFormControlValue,
        resetForm,
        addFormControl,
        removeFormControl,
    };
};
const formHandler = Object.assign(Object.assign(Object.assign({ construct(target, arg) {
        const { controls, value, valid } = new target(...arg);
        return { controls, value, valid };
    },
    get(target, prop) {
        if (prop in target) {
            return target[prop];
        }
        throw new Error(`Invalid property ${prop}`);
    },
    set(_, prop, __) {
        throw new Error(`Cannot set property value of ${prop}`);
    } }, proxykits_1.hasProp), proxykits_1.ownKeyProp), proxykits_1.defineProp);
